"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.changesByIdAfter = exports.changesByIdBefore = void 0;
const feathers_hooks_common_1 = require("feathers-hooks-common");
const cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
const feathers_utils_1 = require("feathers-utils");
const defaultOptions = {
    skipHooks: false,
    refetchItems: false,
    params: undefined
};
const changesById = (cb, providedOptions) => {
    const options = Object.assign({}, defaultOptions, providedOptions);
    return (context) => __awaiter(void 0, void 0, void 0, function* () {
        if (feathers_utils_1.shouldSkip("checkMulti", context)) {
            return context;
        }
        if (context.type === "before") {
            yield exports.changesByIdBefore(context, options);
        }
        else if (context.type === "after") {
            yield exports.changesByIdAfter(context, cb, options);
        }
        return context;
    });
};
const updateMethods = ["update", "patch"];
const changesByIdBefore = (context, options) => __awaiter(void 0, void 0, void 0, function* () {
    let byId;
    if (context.method === "create") {
        byId = {};
    }
    else if (updateMethods.includes(context.method) ||
        context.method === "remove") {
        byId = yield getOrFindById(context, options.params, { skipHooks: options.skipHooks });
    }
    else {
        return context;
    }
    context.params.changelog = context.params.changelog || {};
    context.params.changelog.itemsBefore = byId;
    return context;
});
exports.changesByIdBefore = changesByIdBefore;
const changesByIdAfter = (context, cb, options) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    if (!((_a = context.params.changelog) === null || _a === void 0 ? void 0 : _a.itemsBefore)) {
        return context;
    }
    const { itemsBefore } = context.params.changelog;
    const items = yield resultById(context, options);
    if (!items) {
        return context;
    }
    const itemsBeforeOrAfter = (context.method === "remove") ? itemsBefore : items;
    const changesById = Object.keys(itemsBeforeOrAfter).reduce((result, id) => {
        if ((context.method !== "create" && !itemsBefore[id]) ||
            (context.method !== "remove" && !items[id])) {
            throw new Error("Mismatch!");
            //return result;
        }
        const before = itemsBefore[id];
        const item = items[id];
        result[id] = {
            before: before,
            item: item
        };
        return result;
    }, {});
    context.params.changesById = changesById;
    if (cb && typeof cb === "function") {
        yield cb(changesById, context);
    }
    return context;
});
exports.changesByIdAfter = changesByIdAfter;
const getIdField = (context) => {
    return context.service.options.id;
};
const getOrFindById = (context, makeParams, _options) => __awaiter(void 0, void 0, void 0, function* () {
    var _b;
    const options = _options || Object.assign({
        skipHooks: true,
        byId: true
    }, _options);
    let itemOrItems;
    const idField = getIdField(context);
    if (context.id == null) {
        const method = (options.skipHooks) ? "_find" : "find";
        if (context.type === "before") {
            let params = cloneDeep_1.default(context.params);
            if ((_b = params.query) === null || _b === void 0 ? void 0 : _b.$select) {
                delete params.query.$select;
            }
            params = Object.assign({ paginate: false }, params);
            params = (typeof makeParams === "function") ? yield makeParams(params, context) : context.params;
            itemOrItems = yield context.service[method](params);
        }
        else if (context.type === "after") {
            itemOrItems = feathers_hooks_common_1.getItems(context);
            if (!itemOrItems) {
                return;
            }
            const fetchedItems = (Array.isArray(itemOrItems)) ? itemOrItems : [itemOrItems];
            const ids = fetchedItems.map(x => x && x[idField]);
            let params = (context.id)
                ? {
                    query: {
                        [idField]: { $in: ids }
                    },
                    paginate: false
                }
                : {};
            params = (makeParams) ? yield makeParams(params, context) : params;
            itemOrItems = yield context.service[method](params);
        }
        itemOrItems = itemOrItems && (itemOrItems.data || itemOrItems);
    }
    else {
        const method = (options.skipHooks) ? "_get" : "get";
        const query = Object.assign({}, context.params.query);
        delete query.$select;
        let params = Object.assign({}, context.params, { query });
        params = (typeof makeParams === "function") ? yield makeParams(params, context) : params;
        itemOrItems = yield context.service[method](context.id, params);
    }
    const items = (!itemOrItems)
        ? []
        : (Array.isArray(itemOrItems))
            ? itemOrItems
            : [itemOrItems];
    if (options.byId) {
        return items.reduce((byId, item) => {
            const id = item[idField];
            byId[id] = item;
            return byId;
        }, {});
    }
    else {
        return items;
    }
});
const resultById = (context, options) => __awaiter(void 0, void 0, void 0, function* () {
    var _c;
    if (!context.result) {
        return {};
    }
    let items;
    if (context.method === "remove" || (!options.refetchItems && !((_c = context.params.query) === null || _c === void 0 ? void 0 : _c.$select))) {
        let itemOrItems = context.result;
        itemOrItems = (Array.isArray(itemOrItems.data)) ? itemOrItems.data : itemOrItems;
        items = Array.isArray(itemOrItems) ? itemOrItems : [itemOrItems];
    }
    else {
        items = (yield getOrFindById(context, options.params, { skipHooks: options.skipHooks, byId: false }));
    }
    const idField = context.service.id;
    return items.reduce((byId, item) => {
        const id = item[idField];
        byId[id] = item;
        return byId;
    }, {});
});
exports.default = changesById;
