"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.afterHook = exports.beforeHook = void 0;
const feathers_hooks_common_1 = require("feathers-hooks-common");
const changesById_1 = require("./changesById");
const transformMustache_1 = __importDefault(require("../utils/transformMustache"));
const sift_1 = __importDefault(require("sift"));
const cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
const trigger = (options) => {
    if (!options) {
        throw new Error("You should define subscriptions");
    }
    return (context) => __awaiter(void 0, void 0, void 0, function* () {
        feathers_hooks_common_1.checkContext(context, null, ["create", "update", "patch", "remove"], "trigger");
        if (context.type === "before") {
            return yield exports.beforeHook(context, options);
        }
        else if (context.type === "after") {
            return yield exports.afterHook(context);
        }
    });
};
const beforeHook = (context, options) => __awaiter(void 0, void 0, void 0, function* () {
    let subs = yield getSubscriptions(context, options);
    if (!(subs === null || subs === void 0 ? void 0 : subs.length)) {
        return context;
    }
    if (!Array.isArray(context.data)) {
        subs = subs.reduce((result, sub) => {
            //@ts-expect-error context is not Record<string, unknown>
            const conditionsData = testCondition(context, context.data, sub.conditionsData);
            if (conditionsData === false) {
                return result;
            }
            sub = (conditionsData)
                ? Object.assign({}, sub, { conditionsData })
                : sub;
            result.push(sub);
            return result;
        }, []);
    }
    if (!(subs === null || subs === void 0 ? void 0 : subs.length)) {
        return context;
    }
    yield changesById_1.changesByIdBefore(context, { skipHooks: false });
    setConfig(context, "subscriptions", subs);
    return context;
});
exports.beforeHook = beforeHook;
const afterHook = (context) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const subs = getConfig(context, "subscriptions");
    if (!(subs === null || subs === void 0 ? void 0 : subs.length)) {
        return context;
    }
    yield changesById_1.changesByIdAfter(context, null, {
        skipHooks: false,
        refetchItems: false
    });
    const now = new Date();
    const promises = [];
    const changesById = (_a = context.params) === null || _a === void 0 ? void 0 : _a.changesById;
    if (!changesById) {
        return context;
    }
    const changes = Object.values(changesById);
    for (const change of changes) {
        const { before } = change;
        for (let sub of subs) {
            let { item } = change;
            let changeForSub = change;
            if (sub.conditionsResult || sub.conditionsBefore || sub.params || sub.view) {
                sub = cloneDeep_1.default(sub);
            }
            const { conditionsResult, conditionsBefore } = sub;
            if (sub.params) {
                const params = (typeof sub.params === "function")
                    ? sub.params(change, sub, changes, subs)
                    : sub.params;
                if (params) {
                    const idField = getIdField(context);
                    item = yield context.service.get(item[idField], params);
                    if (item) {
                        changeForSub = Object.assign({}, change, { item });
                    }
                }
            }
            let mustacheView = {
                item,
                before,
                data: context.data,
                id: context.id,
                method: context.method,
                now,
                params: context.params,
                path: context.path,
                service: context.service,
                type: context.type,
                user: context.params,
            };
            if (sub.view) {
                if (typeof sub.view === "function") {
                    mustacheView = yield sub.view(mustacheView, { item: changeForSub, items: changes, subscription: sub, subscriptions: subs, context });
                }
                else {
                    mustacheView = Object.assign(mustacheView, sub.view);
                }
            }
            const conditionsResultNew = testCondition(mustacheView, item, conditionsResult);
            if (conditionsResultNew === false) {
                continue;
            }
            const conditionsBeforeNew = testCondition(mustacheView, before, conditionsBefore);
            if (conditionsBeforeNew === false) {
                continue;
            }
            sub = Object.assign({}, sub, {
                conditionsBefore: conditionsBeforeNew,
                conditionsResult: conditionsResultNew
            });
            const promise = sub.callAction(changeForSub, sub, changes, context);
            if (sub.isBlocking) {
                promises.push(promise);
            }
        }
    }
    yield Promise.all(promises);
    return context;
});
exports.afterHook = afterHook;
function setConfig(context, key, val) {
    context.params.trigger = context.params.trigger || {};
    context.params.trigger[key] = val;
}
function getConfig(context, key) {
    var _a;
    return (_a = context.params.trigger) === null || _a === void 0 ? void 0 : _a[key];
}
const defaultSubscription = {
    callAction: undefined,
    conditionsBefore: undefined,
    conditionsData: undefined,
    conditionsResult: undefined,
    isBlocking: true,
    method: undefined,
    params: undefined,
    service: undefined,
    view: undefined
};
const getSubscriptions = (context, options) => __awaiter(void 0, void 0, void 0, function* () {
    const _subscriptions = (typeof options === "function")
        ? yield options(context)
        : options;
    if (!_subscriptions) {
        return;
    }
    let subscriptions = (Array.isArray(_subscriptions)) ? _subscriptions : [_subscriptions];
    subscriptions = subscriptions.map(x => Object.assign({}, defaultSubscription, x));
    const { path, method } = context;
    return subscriptions.filter(sub => {
        if ((typeof sub.service === "string" && sub.service !== path) ||
            (Array.isArray(sub.service) && !sub.service.includes(path))) {
            return false;
        }
        if ((typeof sub.method === "string" && sub.method !== method) ||
            (Array.isArray(sub.method) && !sub.method.includes(method))) {
            return false;
        }
        return true;
    });
});
const testCondition = (mustacheView, item, conditions) => {
    if (conditions !== undefined && conditions !== true) {
        conditions = transformMustache_1.default(conditions, mustacheView);
    }
    const areConditionsFulFilled = conditions === undefined || conditions === true || sift_1.default(conditions)(item);
    if (!areConditionsFulFilled) {
        return false;
    }
    return conditions;
};
const getIdField = (context) => {
    return context.service.options.id;
};
exports.default = trigger;
