"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const get_1 = __importDefault(require("lodash/get"));
const has_1 = __importDefault(require("lodash/has"));
const set_1 = __importDefault(require("lodash/set"));
const isPlainObject_1 = __importDefault(require("lodash/isPlainObject"));
const transformMustache = (item, view, root, path) => {
    if (!root) {
        root = item;
    }
    if (!path) {
        path = [];
    }
    if (typeof item === "string") {
        if (!item.startsWith("{{") || !item.endsWith("}}")) {
            return item;
        }
        const key = getMustacheKey(item);
        if (key && has_1.default(view, key)) {
            const val = get_1.default(view, key);
            //@ts-expect-error typing of root
            set_1.default(root, path, val);
        }
        return item;
    }
    else if (isPlainObject_1.default(item)) {
        for (const key in item) {
            transformMustache(item[key], view, root, [...path, key]);
        }
        return item;
    }
    else if (Array.isArray(item)) {
        item.forEach((subItem, i) => transformMustache(subItem, view, root, [...path, `${i}`]));
        return item;
    }
    return item;
};
const getMustacheKey = (item) => {
    if ((item.match(/{{/g) || []).length !== 1 &&
        (item.match(/}}/g) || []).length !== 1) {
        return;
    }
    const keys = item.match(/{{\s*[\w.]+\s*}}/g).map(x => x.match(/[\w.]+/)[0]);
    if (keys.length === 1) {
        return keys[0];
    }
    return undefined;
};
/*export const hasMustacheKey = <T>(
  item: T,
  testKey: string,
  root?: unknown,
  path?: string[]
): boolean => {
  if (!root) { root = item; }
  if (!path) { path = []; }
  if (typeof item === "string") {
    if (!item.startsWith("{{") || !item.endsWith("}}")) {
      return false;
    }

    const key = getMustacheKey(item);

    if (key && (key === testKey || key.startsWith(`${testKey}.`))) {
      return true;
    } else {
      return false;
    }
  } else if (_isPlainObject(item)) {
    for (const key in item) {
      const hasKey = hasMustacheKey(item[key], testKey, root, [...path, key]);
      if (hasKey) { return true; }
    }
    return false;
  } else if (Array.isArray(item)) {
    item.forEach((subItem, i) => {
      const hasKey = hasMustacheKey(subItem, testKey, root, [...path, `${i}`]);
      if (hasKey) { return true; }
    });

    return false;
  }

  return false;
};*/
exports.default = transformMustache;
